package my.vaadin.app;

import java.util.List;

import javax.servlet.annotation.WebServlet;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.icons.VaadinIcons;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.shared.ui.ValueChangeMode;
import com.vaadin.ui.Button;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.Grid;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.TextField;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.themes.ValoTheme;

/**
 * This UI is the application entry point. A UI may either represent a browser window 
 * (or tab) or some part of a html page where a Vaadin application is embedded.
 * <p>
 * The UI is initialized using {@link #init(VaadinRequest)}. This method is intended to be 
 * overridden to add component to the user interface and initialize non-component functionality.
 */
@Theme("mytheme")
public class MyUI extends UI {

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	//inject class service
	private CustomerService service = CustomerService.getInstance();
	//tambahkan grid dengan model Customer untuk menampilkan tabel
	private Grid<Customer> grid = new Grid<>(Customer.class);
	//tambah textfield untuk kolom cari
	private TextField filterText = new TextField();
	
	private CustomerForm form = new CustomerForm(this);

	@Override
    protected void init(VaadinRequest vaadinRequest) {
        final VerticalLayout layout = new VerticalLayout();
        
        filterText.setPlaceholder("silahkan cari..");
        //menambahkan pembaca untuk value yang dicari
        filterText.addValueChangeListener(e -> tableCustomer());
        filterText.setValueChangeMode(ValueChangeMode.LAZY);
        
        //tambah button untuk hapus filter
        Button hapusButton = new Button(VaadinIcons.CIRCLE);
        hapusButton.setDescription("Hapus Filter");
        hapusButton.addClickListener(e -> filterText.clear());
        //tambah button untuk tambah data
        
        Button tambahButton = new Button("Tambah Customer");
        tambahButton.setStyleName(ValoTheme.BUTTON_PRIMARY);
        tambahButton.addClickListener(e->{
        	grid.asSingleSelect().clear();
        	form.setCustomer(new Customer());
        });
        
        //cara set yang kolom yg mau ditampilkan
        grid.setColumns("id","firstName", "lastName", "status");
        
        HorizontalLayout main = new HorizontalLayout(grid, form);
        main.setSizeFull();
        grid.setSizeFull();
        main.setExpandRatio(grid, 1);
        
        CssLayout filter = new CssLayout();
        filter.addComponents(filterText, hapusButton);
        filter.setStyleName(ValoTheme.LAYOUT_COMPONENT_GROUP);
        
        HorizontalLayout toolbar = new HorizontalLayout(filter, tambahButton);
        //tambahkan filter dan main ke layout
        layout.addComponents(toolbar,main);
        
        tableCustomer();
        
        setContent(layout);
        
        //membuat form tak terlihat sebelum klik tombol tambah
        form.setVisible(false);
        
        grid.asSingleSelect().addValueChangeListener(event -> {
        	if(event.getValue()==null) {
        		form.setVisible(false);
        	}else {
        		form.setCustomer(event.getValue());
        	}
        });
        
    }
	public void tableCustomer() {
		//fetch data customer dan taruh di grid + tambahkan parameter filterText.getValue supaya search berfungsi
		List<Customer> customerList = service.findAll(filterText.getValue());
        grid.setItems(customerList);
	}

    @WebServlet(urlPatterns = "/*", name = "MyUIServlet", asyncSupported = true)
    @VaadinServletConfiguration(ui = MyUI.class, productionMode = false)
    public static class MyUIServlet extends VaadinServlet {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
    }
}
